/************************************************************************

  A function to decode the MUSE Raster header record

*************************************************************************/

#include "muse.h"

#ifndef H_VECTOR		/* SR 09/25 */
/*#include "vector.h"*/
#endif

#if XVT_CC_PROTO
extern ERRSTATUS char_to_short(unsigned char *record, short *s, short big_endian, long *c);
extern ERRSTATUS char_to_ushort(unsigned char *record, unsigned short *s, short big_endian, long *c);
extern ERRSTATUS char_to_long(unsigned char *record, long *l, short big_endian, long *c);
extern ERRSTATUS char_to_double(unsigned char *record, double *d, short big_endian, long *c);
#endif

#if XVT_CC_PROTO
ERRSTATUS
decode_raster_hdr(unsigned char *record, RASTER * raster)
#else

ERRSTATUS
decode_raster_hdr(record, raster)
    unsigned char  *record;	/* the header record                    */
    RASTER         *raster;	/* the header structure                 */
#endif

{
    long            i;		/* the loop counters                    */
    long            c;
    short           machine;	/* the machine type that created the file */
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_short(record, &machine, big_endian, &c);

    char_to_long(record, &raster->magic, big_endian, &c);

    for (i = 0; i < 30; i++)
	raster->data_hdr.source[i] = record[c++];

    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_hdr.h_units = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_hdr.v_units = enum_value;
    char_to_double(record, &raster->data_hdr.x_res, big_endian, &c);
    char_to_double(record, &raster->data_hdr.y_res, big_endian, &c);
    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_hdr.projection = enum_value;

    char_to_double(record, &raster->data_hdr.center.lon, big_endian, &c);
    char_to_double(record, &raster->data_hdr.center.lat, big_endian, &c);

    char_to_double(record, &raster->data_hdr.extent.left, big_endian, &c);
    char_to_double(record, &raster->data_hdr.extent.right, big_endian, &c);
    char_to_double(record, &raster->data_hdr.extent.bottom, big_endian, &c);
    char_to_double(record, &raster->data_hdr.extent.top, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_hdr.datum.hor_datum = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_hdr.datum.ver_datum = enum_value;

    char_to_double(record, &raster->data_hdr.accuracy.rel_hor, big_endian, &c);
    char_to_double(record, &raster->data_hdr.accuracy.abs_hor, big_endian, &c);
    char_to_double(record, &raster->data_hdr.accuracy.rel_ver, big_endian, &c);
    char_to_double(record, &raster->data_hdr.accuracy.abs_ver, big_endian, &c);

    for (i = 0; i < 3; i++)
	raster->data_hdr.date.year[i] = record[c++];
    for (i = 0; i < 3; i++)
	raster->data_hdr.date.month[i] = record[c++];
    for (i = 0; i < 3; i++)
	raster->data_hdr.date.day[i] = record[c++];
    for (i = 0; i < 3; i++)
	raster->data_hdr.date.hour[i] = record[c++];
    for (i = 0; i < 3; i++)
	raster->data_hdr.date.minute[i] = record[c++];
    for (i = 0; i < 3; i++)
	raster->data_hdr.date.second[i] = record[c++];

    char_to_double(record, &raster->data_hdr.zoom, big_endian, &c);
    char_to_double(record, &raster->data_hdr.data_min, big_endian, &c);
    char_to_double(record, &raster->data_hdr.data_max, big_endian, &c);

    char_to_long(record, &raster->data_matrix.width, big_endian, &c);
    char_to_long(record, &raster->data_matrix.height, big_endian, &c);
    char_to_long(record, &raster->data_matrix.bits_per_pixel, big_endian, &c);
    char_to_long(record, &raster->data_matrix.bytes_per_row, big_endian, &c);
    char_to_long(record, &raster->data_matrix.bytes_per_row, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_matrix.num_type = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    raster->data_matrix.data_type = enum_value;

    char_to_long(record, &ptr_value, big_endian, &c);
    raster->palette = (PALETTE *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    raster->lut = (LUT *) ptr_value;
    return STAT_SUCCESS;
}

/************************************************************************

  A function to decode the MUSE LUT structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_lut(unsigned char *record, LUT * lut)
#else

ERRSTATUS
decode_lut(record, lut)
    unsigned char  *record;	/* the header record                    */
    LUT            *lut;	/* the header structure                 */
#endif

{
    long            i, j;	/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           bool_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &lut->magic, big_endian, &c);

    for (i = 0; i < MAX_LUT; i++)
    {
	char_to_long(record, &lut->lut_desc[i].num_hues, big_endian, &c);
	char_to_long(record, &lut->lut_desc[i].num_shades, big_endian, &c);
	char_to_double(record, &lut->lut_desc[i].data_range[0], big_endian, &c);
	char_to_double(record, &lut->lut_desc[i].data_range[1], big_endian, &c);
	char_to_double(record, &lut->lut_desc[i].hue_range[0], big_endian, &c);
	char_to_double(record, &lut->lut_desc[i].hue_range[1], big_endian, &c);

	for (j = 0; j < 3; j++)
	{
	    char_to_double(record, &lut->lut_desc[i].shade_range[j].sat,
			   big_endian, &c);
	    char_to_double(record, &lut->lut_desc[i].shade_range[j].val,
			   big_endian, &c);
	}
    }

    for (i = 0; i < MAX_COLORS; i++)
	char_to_ushort(record, &lut->map[i], big_endian, &c);
    char_to_short(record, &bool_value, big_endian, &c);
    lut->stretch = bool_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE PALETTE structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_palette(unsigned char *record, PALETTE * palette)
#else

ERRSTATUS
decode_palette(record, palette)
    unsigned char  *record;	/* the header record                    */
    PALETTE        *palette;	/* the header structure                 */
#endif

{
    long            i /* ,j */ ;/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &palette->magic, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    palette->type = enum_value;

    /* if (palette->type == PAL_NUM_BINS) */
    {
	char_to_ushort(record, &palette->bins.r, big_endian, &c);
	char_to_ushort(record, &palette->bins.g, big_endian, &c);
	char_to_ushort(record, &palette->bins.b, big_endian, &c);
    }

    char_to_ushort(record, &palette->first_color, big_endian, &c);
    char_to_ushort(record, &palette->num_colors, big_endian, &c);

    for (i = 0; i < MAX_COLORS; i++)
    {
	char_to_ushort(record, &palette->rgb[i].r, big_endian, &c);
	char_to_ushort(record, &palette->rgb[i].g, big_endian, &c);
	char_to_ushort(record, &palette->rgb[i].b, big_endian, &c);
    }
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE MAP_DOC structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_mapdoc(unsigned char *record, MAP_DOC * mapdoc)
#else

ERRSTATUS
decode_mapdoc(record, mapdoc)
    unsigned char  *record;	/* the header record                    */
    MAP_DOC        *mapdoc;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &mapdoc->magic, big_endian, &c);

    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->mgm = (MGM *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->units = (UNITS *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->lut = (LUT *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->palette = (PALETTE *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->products = (PRODUCTS *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    mapdoc->view = (VIEW *) ptr_value;
    char_to_short(record, &bool_value, big_endian, &c);
    mapdoc->needs_save = bool_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE MGM structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_mgm(unsigned char *record, MGM * mgm)
#else

ERRSTATUS
decode_mgm(record, mgm)
    unsigned char  *record;	/* the header record                    */
    MGM            *mgm;	/* the header structure                 */
#endif

{
    long            i /* ,j */ ;/* the loop counters                    */
    long            c;
     short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;
    double          dummy;
    long            num_tot_params, num_params;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &mgm->magic, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    mgm->projection = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    mgm->hor_datum = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    mgm->ver_datum = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    mgm->config_pref = enum_value;

    char_to_double(record, &mgm->map_center.lon, big_endian, &c);
    char_to_double(record, &mgm->map_center.lat, big_endian, &c);
    char_to_double(record, &mgm->scale_reciprocal, big_endian, &c);
    char_to_double(record, &mgm->pixel_size_on_screen, big_endian, &c);
    char_to_double(record, &mgm->pixel_size_on_ground, big_endian, &c);
    char_to_double(record, &mgm->pixel_size_lon, big_endian, &c);
    char_to_double(record, &mgm->pixel_size_lat, big_endian, &c);

    char_to_double(record, &mgm->dev_extent.left, big_endian, &c);
    char_to_double(record, &mgm->dev_extent.right, big_endian, &c);
    char_to_double(record, &mgm->dev_extent.bottom, big_endian, &c);
    char_to_double(record, &mgm->dev_extent.top, big_endian, &c);
    char_to_double(record, &mgm->dev_ymin, big_endian, &c);
    char_to_double(record, &mgm->dev_ymax, big_endian, &c);

    char_to_long(record, &mgm->xpixels, big_endian, &c);
    char_to_long(record, &mgm->ypixels, big_endian, &c);
    char_to_long(record, &mgm->bit_map_depth, big_endian, &c);

    char_to_double(record, &mgm->geo_extent.left, big_endian, &c);
    char_to_double(record, &mgm->geo_extent.right, big_endian, &c);
    char_to_double(record, &mgm->geo_extent.bottom, big_endian, &c);
    char_to_double(record, &mgm->geo_extent.top, big_endian, &c);
    char_to_double(record, &mgm->pixasp, big_endian, &c);
    char_to_double(record, &mgm->horizontal_accuracy, big_endian, &c);
    char_to_double(record, &mgm->vertical_accuracy, big_endian, &c);

    char_to_short(record, &bool_value, big_endian, &c);
    mgm->user_aspect = bool_value;

    num_tot_params = sizeof(mgm->params) / sizeof(double);
    switch (mgm->projection)
    {

      case PROJ_LINEAR:
	num_params = sizeof(LINEAR_PARAMS) / sizeof(double);
	char_to_double(record, &mgm->params.linear_params.dummy, big_endian, &c);
	break;

      case PROJ_MERCATOR:
	num_params = sizeof(MERCAT_PARAMS) / sizeof(double);
	char_to_double(record, &mgm->params.mercat_params.dummy, big_endian, &c);
	break;

      default:
	num_params = 0;
	break;
    }
    for (i = num_params; i < num_tot_params; i++)
	char_to_double(record, &dummy, big_endian, &c);

    char_to_short(record, &bool_value, big_endian, &c);
    mgm->valid = bool_value;

    char_to_double(record, &mgm->sx, big_endian, &c);
    char_to_double(record, &mgm->sy, big_endian, &c);
    char_to_double(record, &mgm->tx, big_endian, &c);
    char_to_double(record, &mgm->ty, big_endian, &c);

    char_to_short(record, &bool_value, big_endian, &c);
    mgm->is_locked = bool_value;
    return STAT_SUCCESS;
}

/************************************************************************

  A function to decode the MUSE UNITS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_units(unsigned char *record, UNITS * units)
#else

ERRSTATUS
decode_units(record, units)
    unsigned char  *record;	/* the header record                    */
    UNITS          *units;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &units->magic, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    units->scale = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    units->hor_distance = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    units->ver_distance = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    units->area = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    units->volume = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    units->geographic = enum_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE PRODUCTS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_products(unsigned char *record, PRODUCTS * products)
#else

ERRSTATUS
decode_products(record, products)
    unsigned char  *record;	/* the header record                    */
    PRODUCTS       *products;	/* the header structure                 */
#endif

{
    long            i, j;	/* the loop counters                    */
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &products->magic, big_endian, &c);
    char_to_long(record, &products->bit_flags, big_endian, &c);	/* SR 10/09 */
    char_to_short(record, &products->count, big_endian, &c);
    char_to_short(record, &products->order, big_endian, &c);

    char_to_long(record, &ptr_value, big_endian, &c);
    products->basemap = (BASEMAP *) ptr_value;

    for (i = 0; i < MAX_PRODUCTS; i++)
    {
	char_to_short(record, &enum_value, big_endian, &c);
	products->overlays[i].type = enum_value;
	for (j = 0; j < 64; j++)
	    products->overlays[i].name[j] = record[c++];
	char_to_long(record, &ptr_value, big_endian, &c);
	products->overlays[i].product_pointer = (void *) ptr_value;
    }

    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE VIEW structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_view(unsigned char *record, VIEW * view)
#else

ERRSTATUS
decode_view(record, view)
    unsigned char  *record;	/* the header record                    */
    VIEW           *view;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    long            long_value;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &view->magic, big_endian, &c);

    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_xsize = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_ysize = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_xmin = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_ymin = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_xmax = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->bitmap_ymax = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->scroll_x = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->scroll_y = (int) long_value;
    char_to_long(record, &long_value, big_endian, &c);
    view->status = (int) long_value;
    char_to_double(record, &view->zoomx, big_endian, &c);
    char_to_double(record, &view->zoomy, big_endian, &c);
    char_to_long(record, &ptr_value, big_endian, &c);
    view->winstuff = (void *) ptr_value;
    char_to_short(record, &bool_value, big_endian, &c);
    view->new_palette = bool_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE DEFAULTS structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_defaults(unsigned char *record, DEFAULTS * defaults)
#else

ERRSTATUS
decode_defaults(record, defaults)
    unsigned char  *record;	/* the header record                    */
    DEFAULTS       *defaults;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &defaults->magic, big_endian, &c);

    char_to_long(record, &defaults->bitmap_matrix.width, big_endian, &c);
    char_to_long(record, &defaults->bitmap_matrix.height, big_endian, &c);
    char_to_long(record, &defaults->bitmap_matrix.bits_per_pixel, big_endian, &c);
    char_to_long(record, &defaults->bitmap_matrix.bytes_per_row, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    defaults->bitmap_matrix.num_type = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    defaults->bitmap_matrix.data_type = enum_value;
    char_to_short(record, &enum_value, big_endian, &c);
    defaults->color_pref = enum_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE BASEMAP structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_basemap(unsigned char *record, BASEMAP * basemap)
#else

ERRSTATUS
decode_basemap(record, basemap)
    unsigned char  *record;	/* the header record                    */
    BASEMAP        *basemap;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &basemap->magic, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    basemap->product = enum_value;

    char_to_long(record, &ptr_value, big_endian, &c);
    basemap->area = (RASTER *) ptr_value;
    char_to_long(record, &ptr_value, big_endian, &c);
    basemap->bit_map = (RASTER *) ptr_value;

    char_to_double(record, &basemap->render_info.shade_threshold, big_endian, &c);
    char_to_double(record, &basemap->render_info.precision, big_endian, &c);
    char_to_long(record, &basemap->render_info.sunx, big_endian, &c);
    char_to_long(record, &basemap->render_info.suny, big_endian, &c);

    char_to_double(record, &basemap->zoom, big_endian, &c);

    char_to_short(record, &bool_value, big_endian, &c);
    basemap->needs_refresh = bool_value;
    char_to_short(record, &bool_value, big_endian, &c);
    basemap->needs_redraw = bool_value;
    char_to_short(record, &bool_value, big_endian, &c);
    basemap->needs_reload = bool_value;
    char_to_short(record, &bool_value, big_endian, &c);
    basemap->warp = bool_value;
    return STAT_SUCCESS;

}

/************************************************************************

  A function to decode the MUSE GRAT structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_grat(unsigned char *record, GRAT_DATA * grat, long version)
#else

ERRSTATUS
decode_grat(record, grat, version)
    unsigned char  *record;	/* the header record                    */
    GRAT_DATA      *grat;	/* the header structure                 */
    long            version;
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           enum_value;
    short           bool_value;

    /* Check for endian type */

    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;

    char_to_long(record, &grat->color, big_endian, &c);

    /*
     * SR 10/09.  If we drop support for old .map versions, remove version
     * from argument list and this if, but keep decoding for magic and
     * bit_flags
     */
    if (version > 0)
    {
	char_to_long(record, &grat->magic, big_endian, &c);
	char_to_long(record, &grat->bit_flags, big_endian, &c);
    }
    char_to_short(record, &bool_value, big_endian, &c);
    grat->contains_valid_data = bool_value;
    /* border major */
    char_to_short(record, &bool_value, big_endian, &c);
    grat->border_enable = bool_value;

    char_to_short(record, &bool_value, big_endian, &c);
    grat->border_major_enable = bool_value;

    char_to_double(record, &grat->border_major_lat, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_major_lat_unit = enum_value;

    char_to_double(record, &grat->border_major_lon, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_major_lon_unit = enum_value;

    char_to_long(record, &grat->border_major_color, big_endian, &c);



    /* border minor */
    char_to_short(record, &bool_value, big_endian, &c);
    grat->border_minor_enable = bool_value;

    char_to_double(record, &grat->border_minor_lat, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_minor_lat_unit = enum_value;

    char_to_double(record, &grat->border_minor_lon, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_minor_lon_unit = enum_value;

    char_to_long(record, &grat->border_minor_color, big_endian, &c);



    /* border label */
    char_to_short(record, &bool_value, big_endian, &c);
    grat->border_labels_enable = bool_value;

    char_to_double(record, &grat->border_labels_lat, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_labels_lat_unit = enum_value;

    char_to_double(record, &grat->border_labels_lon, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->border_labels_lon_unit = enum_value;

    char_to_long(record, &grat->border_labels_color, big_endian, &c);



    /* grat major */
    char_to_short(record, &bool_value, big_endian, &c);
    grat->grat_enable = bool_value;

    char_to_short(record, &bool_value, big_endian, &c);
    grat->grat_major_enable = bool_value;

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_major = enum_value;

    grat->grat_major_ascii_code = record[c++];

    char_to_double(record, &grat->grat_major_lat, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_major_lat_unit = enum_value;

    char_to_double(record, &grat->grat_major_lon, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_major_lon_unit = enum_value;

    char_to_long(record, &grat->grat_major_color, big_endian, &c);


    /* grat_minor */
    char_to_short(record, &bool_value, big_endian, &c);
    grat->grat_minor_enable = bool_value;

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_minor = enum_value;

    grat->grat_minor_ascii_code = record[c++];

    char_to_double(record, &grat->grat_minor_lat, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_minor_lat_unit = enum_value;

    char_to_double(record, &grat->grat_minor_lon, big_endian, &c);

    char_to_short(record, &enum_value, big_endian, &c);
    grat->grat_minor_lon_unit = enum_value;

    char_to_long(record, &grat->grat_minor_color, big_endian, &c);

    return STAT_SUCCESS;
}
#if 0
/************************************************************************

  A function to decode the MUSE VEC_DATA structure

*************************************************************************/

#if XVT_CC_PROTO
ERRSTATUS
decode_vec_data(unsigned char *record, VEC_DATA * vec_data)
#else

ERRSTATUS
decode_vec_data(record, vec_data)
    unsigned char  *record;	/* the header record                    */
    VEC_DATA       *vec_data;	/* the header structure                 */
#endif

{
    long            c;
    short           end_check = 1;
    char           *end_char;
    short           big_endian;
    short           bool_value;
    long            ptr_value;

    /* Check for endian type */
    end_char = (char *) &end_check;
    big_endian = 1 - (short) (*end_char);

    /* Decode the values */

    c = 0;
    /* SR 10/09 */
    char_to_long(record, &vec_data->magic, big_endian, &c);
    char_to_long(record, &vec_data->bit_flags, big_endian, &c);

    char_to_short(record, &bool_value, big_endian, &c);
    vec_data->contains_valid_data = bool_value;

    char_to_double(record, &vec_data->edge_width, big_endian, &c);

    char_to_long(record, &vec_data->edge_color, big_endian, &c);

    char_to_long(record, &vec_data->text_color, big_endian, &c);

    char_to_long(record, &vec_data->point_color, big_endian, &c);

    char_to_long(record, &ptr_value, big_endian, &c);
    vec_data->vector = (VECTOR *) ptr_value;

    return STAT_SUCCESS;
}
#endif
