/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * MapKit definitions
 */

#include <MapUnidraw/gseditor.h>
#include <MapUnidraw/gsview.h>
#include <MapUnidraw/thchooser.h>
#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapimport.h>
#include <MapUnidraw/mappoint.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>
#include <MapUnidraw/mapviewer.h>
#include <MapUnidraw/mapviews.h>
#include <MapUnidraw/mapviewstate.h>

#include <Map/mapcov.h>
#include <Map/mapdb.h>
#include <Map/mapelt.h>
#include <Map/mapfclass.h>
#include <Map/maplib.h>

#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mapkit.h>

#include <OverlayUnidraw/annotate.h>
#include <OverlayUnidraw/ovabout.h>
#include <OverlayUnidraw/ovarrow.h>
#include <OverlayUnidraw/ovctrl.h>
#include <OverlayUnidraw/ovcamcmds.h>
#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovdoer.h>
#include <OverlayUnidraw/ovellipse.h>
#include <OverlayUnidraw/ovpage.h>
#include <OverlayUnidraw/ovpolygon.h>
#include <OverlayUnidraw/ovprint.h>
#include <OverlayUnidraw/ovrect.h>
#include <OverlayUnidraw/ovspline.h>
#include <OverlayUnidraw/ovtext.h>

#include <UniIdraw/idkybd.h>

#include <Unidraw/unidraw.h>
#include <Unidraw/editor.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/grid.h>
#include <Unidraw/uctrls.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/catalog.h>
#include <Unidraw/viewer.h>

#include <Unidraw/Commands/catcmds.h>

#include <Unidraw/Components/text.h>

#include <Unidraw/Graphic/ellipses.h>
#include <Unidraw/Graphic/polygons.h>

#include <Unidraw/Tools/connect.h>
#include <Unidraw/Tools/grcomptool.h>
#include <Unidraw/Tools/magnify.h>
#include <Unidraw/Tools/move.h>
#include <Unidraw/Tools/reshape.h>
#include <Unidraw/Tools/rotate.h>
#include <Unidraw/Tools/scale.h>
#include <Unidraw/Tools/select.h>
#include <Unidraw/Tools/stretch.h>

#define iv2_6_compatible 1
#include <UniIdraw/idarrows.h>
#undef iv2_6_compatible

#include <InterViews/frame.h>
#include <IV-look/kit.h>
#include <InterViews/background.h>
#include <InterViews/layout.h>
#include <InterViews/label.h>
#include <InterViews/action.h>
#include <InterViews/style.h>
#include <InterViews/target.h>
#include <InterViews/window.h>

#include <InterViews/display.h>
#include <InterViews/session.h>
#include <IV-X11/xdisplay.h>
#include <IV-X11/xcolor.h>
#include <IV-X11/xpattern.h>

#include <OS/math.h>

#include <IVGlyph/figure.h>
#include <IVGlyph/toolbutton.h>
#include <IVGlyph/exportchooser.h>
#include <IVGlyph/importchooser.h>

/******************************************************************************/

static const unit = 15;

static int xClosed[] = { unit/5, unit, unit, unit*3/5, 0 };
static int yClosed[] = { 0, unit/5, unit*3/5, unit, unit*2/5 };
static Coord fxClosed[] = { unit/5, unit, unit, unit*3/5, 0 };
static Coord fyClosed[] = { 0, unit/5, unit*3/5, unit, unit*2/5 };
static const int nClosed = 5;

static int xOpen[] = { 0, unit/2, unit/2, unit };
static int yOpen[] = { 0, unit/4, unit*3/4, unit };
static Coord fxOpen[] = { 0, unit/2, unit/2, unit };
static Coord fyOpen[] = { 0, unit/4, unit*3/4, unit };
static const int nOpen = 4;

static Coord xhead[] = { unit, unit-4, unit-7 };
static Coord yhead[] = { unit, unit-7, unit-4 };

static const char* page_width_attrib = "pagewidth";
static const char* page_height_attrib = "pageheight";
static const char* page_cols_attrib = "pagecols";
static const char* page_rows_attrib = "pagerows";
static const char* grid_x_incr = "gridxincr";
static const char* grid_y_incr = "gridyincr";

/******************************************************************************/

implementActionCallback(ShowHideDoer)

ShowHideDoer::ShowHideDoer(Command* s, Command* h, TelltaleState* st) {
    scmd = s;
    hcmd = h;
    state = st;
}

void ShowHideDoer::Do() {
    Command* command;
    if (state->test(TelltaleState::is_chosen))
	command = scmd;
    else
	command = hcmd;
    if (command) {
	if (command->Reversible()) {
	    command = command->Copy();
	    command->Execute();
            if (command->Reversible()) {
                command->Log();
            } else {
                delete command;
            }
	}
	else {
	    command->Execute();
            if (command->Reversible()) {
                command = command->Copy();
                command->Log();
            }
	}
    }
}

/******************************************************************************/

MapKit* MapKit::_mapkit = nil;

MapKit::MapKit ()
:OverlayKit()
{
  _projnameview = nil;
}

MapKit* MapKit::Instance() {
    if (!_mapkit)
	_mapkit = new MapKit();
    return _mapkit;
}

Glyph* MapKit::MakeToolbar() {
    WidgetKit& kit = *WidgetKit::instance();
    const LayoutKit& layout = *LayoutKit::instance();
    Style* s = kit.style();

    PolyGlyph* vb = layout.vbox(15);

    TelltaleGroup* tg = new TelltaleGroup();

    Glyph* sel = kit.label("Select");
    Glyph* mov = kit.label("Move");
    Glyph* scl = kit.label("Scale");
    Glyph* str = kit.label("Stretch");
    Glyph* rot = kit.label("Rotate");
    Glyph* alt = kit.label("Alter");
    Glyph* mag = kit.label("Magnify");
    Glyph* txt = kit.label("Text");
    Glyph* glin = new Fig31Line(new Brush(0), kit.foreground(), nil,
				0, 0, unit, unit);
    Glyph* gmlin = new Fig31Polyline(new Brush(0), kit.foreground(), nil,
				     fxOpen, fyOpen, nOpen);
    Glyph* gospl = new Fig31Open_BSpline(new Brush(0), kit.foreground(), nil,
					 fxOpen, fyOpen, nOpen);
    Glyph* grect = new Fig31Rectangle(new Brush(0), kit.foreground(), nil,
				      0, 0, unit, unit*4/5);
    Glyph* gellp = new Fig31Ellipse(new Brush(0), kit.foreground(), nil,
				    0, 0, unit*2/3, unit*2/5);
    Glyph* gpoly = new Fig31Polygon(new Brush(0), kit.foreground(), nil,
				    fxClosed, fyClosed, nClosed);
    Glyph* gcspl = new Fig31Closed_BSpline(new Brush(0), kit.foreground(), nil,
					   fxClosed, fyClosed, nClosed);
    Glyph* anno = kit.label("Annotate");
    Glyph* ins = kit.label("Inspect");

    Coord maxwidth = 0;
    Requisition req;
    maxwidth = Math::max((mag->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((sel->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((mov->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((scl->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((str->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((rot->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((alt->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((mag->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((txt->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((glin->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((gmlin->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((gospl->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((grect->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((gellp->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((gpoly->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((gcspl->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((anno->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((ins->request(req), req.x_requirement().natural()),
			 maxwidth);

    vb->append(MakeTool(new SelectTool(new ControlInfo("Select", KLBL_SELECT, CODE_SELECT)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(sel)),
			tg));
    vb->append(MakeTool(new MoveTool(new ControlInfo("Move", KLBL_MOVE, CODE_MOVE)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(mov)),
			tg));
    vb->append(MakeTool(new ScaleTool(new ControlInfo("Scale", KLBL_SCALE, CODE_SCALE)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(scl)), tg));
    vb->append(MakeTool(new StretchTool(new ControlInfo("Stretch", KLBL_STRETCH,CODE_STRETCH)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(str)), tg));
    vb->append(MakeTool(new RotateTool(new ControlInfo("Rotate", KLBL_ROTATE, CODE_ROTATE)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(rot)), tg));
    vb->append(MakeTool(new ReshapeTool(new ControlInfo("Alter", KLBL_RESHAPE, CODE_RESHAPE)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(alt)), tg));
    vb->append(MakeTool(new MagnifyTool(new ControlInfo("Magnify", KLBL_MAGNIFY,CODE_MAGNIFY)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(mag)), tg));
    TextGraphic* text = new TextGraphic("Text", stdgraphic);
    TextOvComp* textComp = new TextOvComp(text);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo("Text", KLBL_TEXT, CODE_TEXT), textComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(txt)), tg));
    ArrowLine* line = new ArrowLine(
	0, 0, unit, unit, false, false, 1., stdgraphic
    );
    ArrowLineOvComp* arrowLineComp = new ArrowLineOvComp(line);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(arrowLineComp, KLBL_LINE, CODE_LINE), arrowLineComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(glin)),
			tg));
    ArrowMultiLine* ml = new ArrowMultiLine(
        xOpen, yOpen, nOpen, false, false, 1., stdgraphic
    );
    ml->SetPattern(psnonepat);
    ArrowMultiLineOvComp* mlComp = new ArrowMultiLineOvComp(ml);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(mlComp, KLBL_MULTILINE, CODE_MULTILINE), mlComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(gmlin)),
			tg));
    ArrowOpenBSpline* spl = new ArrowOpenBSpline(
        xOpen, yOpen, nOpen, false, false, 1., stdgraphic
    );
    spl->SetPattern(psnonepat);
    ArrowSplineOvComp* splComp = new ArrowSplineOvComp(spl);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(splComp, KLBL_SPLINE, CODE_SPLINE), splComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(gospl)),
			tg));
    SF_Rect* rect = new SF_Rect(0, 0, unit, unit*4/5, stdgraphic);
    rect->SetPattern(psnonepat);
    RectOvComp* rectComp = new RectOvComp(rect);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(rectComp, KLBL_RECT, CODE_RECT), rectComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(grect)),
			tg));
    SF_Ellipse* ellipse = new SF_Ellipse(0, 0, unit*2/3, unit*2/5, stdgraphic);
    ellipse->SetPattern(psnonepat);
    EllipseOvComp* ellipseComp = new EllipseOvComp(ellipse);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(ellipseComp, KLBL_ELLIPSE, CODE_ELLIPSE), ellipseComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(gellp)),
			tg));
    SF_Polygon* polygon = new SF_Polygon(xClosed, yClosed, nClosed,stdgraphic);
    polygon->SetPattern(psnonepat);
    PolygonOvComp* polygonComp = new PolygonOvComp(polygon);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(polygonComp, KLBL_POLY, CODE_POLY), polygonComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(gpoly)),
			tg));
    SFH_ClosedBSpline* cspline = new SFH_ClosedBSpline(
        xClosed, yClosed, nClosed, stdgraphic
    );
    cspline->SetPattern(psnonepat);
    ClosedSplineOvComp* csplineComp = new ClosedSplineOvComp(cspline);
    vb->append(MakeTool(new GraphicCompTool(new ControlInfo(csplineComp, KLBL_CSPLINE,CODE_CSPLINE), csplineComp),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(gcspl)),
			tg));
    vb->append(MakeTool(new AnnotateTool(new ControlInfo("Annotate", "A", "A")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(anno)), tg));
    vb->append(MakeTool(new MapInspectTool(((MapEditor*)_ed)->selstate(),
					   new ControlInfo("Inspect", "i", "i")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(ins)), tg));

    return layout.hbox(
	layout.vflexible(
	    layout.vnatural(
		new Background(
		    layout.vcenter(
			vb
		    ),
		    unidraw->GetCatalog()->FindColor("#aaaaaa")
		),
		550
	    )
	)
    );
}

Glyph* MapKit::MakeStates() {
    Catalog* catalog = unidraw->GetCatalog();
    MapDatabase* mapdb = ((MapEditor*)_ed)->_mapdb;

    ((MapEditor*)_ed)->_fclassstate = new MapFClassState(catalog->GetAttribute("fclass"),
	mapdb ? 
	(mapdb->library(catalog->GetAttribute("library")) ?
           mapdb->library(catalog->GetAttribute("library"))->
	coverage(catalog->GetAttribute("coverage"))->
	feature_class(catalog->GetAttribute("fclass"))->description():"") : "");

    ((MapEditor*)_ed)->_covstate = new MapCovState(((MapEditor*)_ed)->_fclassstate,
	catalog->GetAttribute("coverage"),
	mapdb ? (mapdb->library(catalog->GetAttribute("library"))?mapdb->library(catalog->GetAttribute("library"))->
	coverage(catalog->GetAttribute("coverage"))->description():"") : "");

    ((MapEditor*)_ed)->_libstate = new MapLibState(((MapEditor*)_ed)->_covstate, mapdb && mapdb->library(catalog->GetAttribute("library")) ? catalog->GetAttribute("library") : "");
    ((MapEditor*)_ed)->_dbstate = new MapDbState(((MapEditor*)_ed)->_libstate, mapdb && strcmp(mapdb->name(), catalog->GetAttribute("database"))==0 ? catalog->GetAttribute("database") : (mapdb ? mapdb->name() : ""));
    ((MapEditor*)_ed)->_projstate = new ProjectionState();
    ((MapEditor*)_ed)->_selstate = new MapSelectionState("");

    NameView* fclassview = new NameView(((MapEditor*)_ed)->_fclassstate);
    NameView* covview = new NameView(((MapEditor*)_ed)->_covstate);
    NameView* libview = new NameView(((MapEditor*)_ed)->_libstate);
    NameView* dbview = new NameView(((MapEditor*)_ed)->_dbstate);
    NameView* projview = new NameView(((MapEditor*)_ed)->_projstate);
    NameView* selview = new NameView(((MapEditor*)_ed)->_selstate);

    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    _projnameview = projview;
    return kit.outset_frame(
	lk.hmargin(
	    lk.vbox(
		lk.vmargin(lk.hbox(dbview, projview), 2),
		lk.vmargin(libview, 2),
		lk.vmargin(selview, 2)
	    ),
	    4,0,0,0.0,fil,0.0
	)
    );
}

void MapKit::InitViewer () {
    Catalog* catalog = unidraw->GetCatalog();

    const char* page_w = catalog->GetAttribute(page_width_attrib);
    const char* page_h = catalog->GetAttribute(page_height_attrib);
    const char* page_cols = catalog->GetAttribute(page_cols_attrib);
    const char* page_rows = catalog->GetAttribute(page_rows_attrib);
    const char* x_incr = catalog->GetAttribute(grid_x_incr);
    const char* y_incr = catalog->GetAttribute(grid_y_incr);

    GraphicView* view = (GraphicView*)_ed->GetComponent()->Create(COMPONENT_VIEW);
    _ed->GetComponent()->Attach(view);
    ((MapIdrawView*)view)->SetMapEditor((MapEditor*)_ed);
    view->Update();

    float w = round(atof(page_w) * ivinches);
    float h = round(atof(page_h) * ivinches);
    if (page_cols && page_rows) {
      int ncols = atoi(page_cols);
      int nrows = atoi(page_rows);
      if (ncols>0 && nrows>0) {
	w = ncols;
	h = nrows;
      }
    }

    OverlayPage* page = MapViewer::make_map_page((OverlayComp*)_ed->GetComponent());
    if (!page) page = new OverlayPage(w, h);
    Grid* grid = new Grid(w, h, atof(x_incr), atof(y_incr));
    grid->Visibility(false);

    MapViewer* viewer = new MapViewer(_ed, view, page, grid); 
    _ed->SetViewer(viewer);
    viewer->scribble_pointer(true);

}

Interactor* MapKit::Interior() {
    return ((MapEditor*)_ed)->Interior();
}

void MapKit::InitLayout(const char* name) {
    if (_ed->GetWindow() == nil) {
	const LayoutKit& layout = *LayoutKit::instance();
	Glyph* states = MakeStates();
	Glyph* menus = MakeMenus();
	Glyph* toolbar = MakeToolbar();
	if (states)
	    menus->append(states);
	Target* viewer = 
	    new Target(Interior(), TargetPrimitiveHit);
	toolbar->append(layout.vcenter(viewer));
	menus->append(toolbar);
	ManagedWindow* w = new ApplicationWindow(_ed);
	_ed->SetWindow(w);
	Style* s = new Style(Session::instance()->style());
	s->alias(name);
	w->style(s);
	_ed->body(menus);
	_ed->GetKeyMap()->Execute("i");
#if 0
	_gseditor = new GSEditor(unidraw->GetCatalog(),
				 new ObservableGS(stdgraphic),
				 ((MapEditor*)GetEditor()));
	_gseditor->ref();
	_thchooser = new ThemeChooser(Session::instance()->style());
	_thchooser->ref();
#else
	_gseditor = nil;
	_thchooser = nil;
#endif
    }
}

Glyph* MapKit::MakeMenus()
{
    LayoutKit& lk = *LayoutKit::instance();
    Menu* menubar_ = WidgetKit::instance()->menubar();
    
    menubar_->append_item(MakeFileMenu());
    menubar_->append_item(MakeEditMenu());
    menubar_->append_item(MakeStructureMenu());
    menubar_->append_item(MakeFontMenu());
    menubar_->append_item(MakeBrushMenu());
    menubar_->append_item(MakePatternMenu());
    menubar_->append_item(MakeFgColorMenu());
    menubar_->append_item(MakeBgColorMenu());
    menubar_->append_item(MakeAlignMenu());
    menubar_->append_item(MakeViewMenu());
    Resource::ref(menubar_);
    return LayoutKit::instance()->vbox(lk.hnatural(menubar_, 700));
}

MenuItem * MapKit::MakeFileMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("File"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, new MapAboutCmd(new ControlInfo("About utmviewer", KLBL_ABOUT, CODE_ABOUT)),
	     "About utmviewer   ");
    MakeMenu(mbi, new OvNewCompCmd(new ControlInfo("New", KLBL_NEWCOMP, CODE_NEWCOMP),
				 new MapIdrawComp),
	     "New   ");
    MakeMenu(mbi, new OvRevertCmd(new ControlInfo("Revert", KLBL_REVERT, CODE_REVERT)),
	     "Revert   ");
    MakeMenu(mbi, new MapOpenCmd(new ControlInfo("Open...", KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open...   ");
    MakeMenu(mbi, new OvSaveCompCmd(new ControlInfo("Save", KLBL_SAVECOMP, CODE_SAVECOMP)),
	     "Save   ");
    MakeMenu(mbi, new OvSaveCompAsCmd(new ControlInfo("Save As...",
					    KLBL_SAVECOMPAS,
					    CODE_SAVECOMPAS)),
	     "Save As...   ");
    MakeMenu(mbi, new OvPrintCmd(new ControlInfo("Print...", KLBL_PRINT, CODE_PRINT)),
	     "Print...   ");
    Style * istyle = new Style(Session::instance()->style());
    istyle->attribute("subcaption", "Import map features from file:");
    istyle->attribute("open", "Import");
    ImportChooser* ichooser = new ImportChooser(".", WidgetKit::instance(), istyle, nil, false, true);
    Resource::ref(ichooser);
    MakeMenu(mbi, new MapImportCmd(new ControlInfo("Import Map Features...",
						  KLBL_IMPORT,
						  CODE_IMPORT),
				   ichooser),
	     "Import Map Features...   ");
    Style * estyle = new Style(Session::instance()->style());
    estyle->attribute("subcaption", "Export selected map features to file:");
    estyle->attribute("open", "Export");
    const char *formats[] = {"idraw", "drawtool"};
    ExportChooser* echooser = new ExportChooser(".", WidgetKit::instance(), estyle,
				 formats, sizeof(formats)/sizeof(char*), nil, nil, true, false);
    Resource::ref(echooser);
    MakeMenu(mbi, new MapExportCmd(new ControlInfo("Export Map Features...",
						  "^X", "\030"),
			       echooser),
	     "Export Map Features...   ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new OvQuitCmd(new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)),
	     "Quit   ");

    return mbi;
}

MenuItem* MapKit::MakeExportMenu() {
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* exportitem = kit.menu_item(kit.label(   "Export to         "));
    Menu* exportmenu = kit.pullright();
    exportitem->menu(exportmenu);

    MapExportCmd* cmd;
    MenuItem* menuitem;
    CommandPusher* pusher;

    menuitem = kit.menu_item(kit.label("mapdraw"));
    cmd = new MapExportCmd(new ControlInfo(""));
    cmd->SetEditor(GetEditor()); 
    pusher = new CommandPusher(cmd); 
    menuitem->action(new ActionCallback(CommandPusher)
		     (pusher, &CommandPusher::Push));
    exportmenu->append_item(menuitem);

    menuitem = kit.menu_item(kit.label("drawtool"));
    cmd = new MapExportCmd(new ControlInfo(""), nil);
    cmd->SetEditor(GetEditor()); 
    pusher = new CommandPusher(cmd); 
    menuitem->action(new ActionCallback(CommandPusher)
		     (pusher, &CommandPusher::Push));
    exportmenu->append_item(menuitem);

    return exportitem;
}

MenuItem* MapKit::MakeViewMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    Style * style = new Style(Session::instance()->style());
    
    MenuItem* mbi = kit.menubar_item(kit.label("View"));
    mbi->menu(kit.pulldown());

#if 0
    MakeMenu(mbi, new OpenDemViewCmd(new ControlInfo("DEM View")),
	     "DEM View      ");
    mbi->menu()->append_item(kit.menu_item_separator());
#endif
    MakeMenu(mbi, new PageCmd(new ControlInfo("Page on/off",
					      "p", "p")),
	     "Page on/off   ");
    MakeMenu(mbi, new PrecisePageCmd(new ControlInfo("Precise Page",
					     "^P", "\020")),
	     "Precise Page   ");
    MakeMenu(mbi, new NormSizeCmd(new ControlInfo("Normal Size",
					  KLBL_NORMSIZE, CODE_NORMSIZE)),
	     "Normal Size   ");
    MakeMenu(mbi, new RedToFitCmd(new ControlInfo("Reduce to Fit",
					  KLBL_REDTOFIT, CODE_REDTOFIT)),
	     "Reduce to Fit   ");
    MakeMenu(mbi, new CenterCmd(new ControlInfo("Center Page",
					KLBL_CENTER, CODE_CENTER)),
	     "Center Page   ");
    MakeMenu(mbi, new OrientationCmd(new ControlInfo("Orientation",
					     KLBL_ORIENTATION,
					     CODE_ORIENTATION)),
	     "Orientation   ");

    mbi->menu()->append_item(kit.menu_item_separator());

    MenuItem* zoomi = kit.menu_item(kit.label("Zoom             "));
    Menu* zoom = kit.pullright();
    zoomi->menu(zoom);
    MakeMenu(zoomi, new ZoomCmd(new ControlInfo("Zoom In"), 2.0),
	     "Zoom In          ");
    MakeMenu(zoomi, new ZoomCmd(new ControlInfo("Zoom Out"), 0.5),
	     "Zoom Out         ");
    MakeMenu(zoomi, new PreciseZoomCmd(new ControlInfo("Precise Zoom")),
	     "Precise Zoom     ");
    mbi->menu()->append_item(zoomi);

    MenuItem* spani = kit.menu_item(kit.label("Small Pan        "));
    Menu* span = kit.pullright();
    spani->menu(span);
    MakeMenu(spani, new FixedPanCmd(new ControlInfo("Small Pan Up"), NO_PAN, PLUS_SMALL_PAN),
	     "Small Pan Up     ");
    MakeMenu(spani, new FixedPanCmd(new ControlInfo("Small Pan Down"), NO_PAN, MINUS_SMALL_PAN),
	     "Small Pan Down   ");
    MakeMenu(spani, new FixedPanCmd(new ControlInfo("Small Pan Left"), MINUS_SMALL_PAN, NO_PAN),
	     "Small Pan Left   ");
    MakeMenu(spani, new FixedPanCmd(new ControlInfo("Small Pan Right"), PLUS_SMALL_PAN, NO_PAN),
	     "Small Pan Right  ");
    mbi->menu()->append_item(spani);

    MenuItem* lpani = kit.menu_item(kit.label("Large Pan        "));
    Menu* lpan = kit.pullright();
    lpani->menu(lpan);
    MakeMenu(lpani, new FixedPanCmd(new ControlInfo("Large Pan Up"), NO_PAN, PLUS_LARGE_PAN),
	     "Large Pan Up     ");
    MakeMenu(lpani, new FixedPanCmd(new ControlInfo("Large Pan Down"), NO_PAN, MINUS_LARGE_PAN),
	     "Large Pan Down   ");
    MakeMenu(lpani, new FixedPanCmd(new ControlInfo("Large Pan Left"), MINUS_LARGE_PAN, NO_PAN),
	     "Large Pan Left   ");
    MakeMenu(lpani, new FixedPanCmd(new ControlInfo("Large Pan Right"), PLUS_LARGE_PAN, NO_PAN),
	     "Large Pan Right  ");
    mbi->menu()->append_item(lpani);

    MakeMenu(mbi, new PrecisePanCmd(new ControlInfo("Precise Pan")),
	     "Precise Pan      ");

    return mbi;
}


void MapKit::InitMembers (OverlayComp* comp) {
  OverlayKit::InitMembers(comp);
  ((MapEditor*)_ed)->AdjustForGeo();
}

